#!/usr/bin/sed -nrf

:_Problem
	# maze generator
	# https://cg.esolangs.gay/32/
:_Restrictions
	# Fixed 9x9 size. At this size most mazes will have a trivial solution.
	# Generation is astronomically slow with own RNG. I can use smaller values in LCG, but it forms loops quickly. Default: system RNG.
	# Initial seed passed to script must be [0-9], for convenience. With system RNG you still might have to wait 5-10 minutes to complete.


b debug
:main
	1h
#9x9 maze setup, each cell is numbered
	s:.*: <SEQ>0-80#result_sp_m<QES> :
	b seq_pos
	:result_sp_m
		s:<SEQ>([^#]+)[^<]+<QES>:\1:
	y: :#:
	s:#(8|17|26|35|44|53|62|71|80)#:&\n###################\n#:g
	s:^:###################\n:
	s:\n#$::

#initial [0-9] seed and LCG(19,18,129)? setup
	x
	1{
		s:^.*(.)$:\1:
		/^[0-9]$/!{
			s:.*:Please give as input a seed between 0 and 9!:p
			b EOS
		}
		s:^[0-9]$:<LCG>& 19 18 129<GCL> 19 18 129:
	}

#adjacency table (no time to calculate it, especially for the sides; top side would even run in negative numbers, not supported atm by my lib)
	s:$:<ADJ>,0-D9R1,1-D10L0R2,2-D11L1R3,3-D12L2R4,4-D13L3R5,5-D14L4R6,6-D15L5R7,7-D16L6R8,8-D17L7,9-U0D18R10,10-U1D19L9R11,11-U2D20L10R12,12-U3D21L11R13,13-U4D22L12R14,14-U5D23L13R15,15-U6D24L14R16,16-U7D25L15R17,17-U8D26L16,18-U9D27R19,19-U10D28L18R20,20-U11D29L19R21,21-U12D30L20R22,22-U13D31L21R23,23-U14D32L22R24,24-U15D33L23R25,25-U16D34L24R26,26-U17D35L25,27-U18D36R28,28-U19D37L27R29,29-U20D38L28R30,30-U21D39L29R31,31-U22D40L30R32,32-U23D41L31R33,33-U24D42L32R34,34-U25D43L33R35,35-U26D44L34,36-U27D45R37,37-U28D46L36R38,38-U29D47L37R39,39-U30D48L38R40,40-U31D49L39R41,41-U32D50L40R42,42-U33D51L41R43,43-U34D52L42R44,44-U35D53L43,45-U36D54R46,46-U37D55L45R47,47-U38D56L46R48,48-U39D57L47R49,49-U40D58L48R50,50-U41D59L49R51,51-U42D60L50R52,52-U43D61L51R53,53-U44D62L52,54-U45D63R55,55-U46D64L54R56,56-U47D65L55R57,57-U48D66L56R58,58-U49D67L57R59,59-U50D68L58R60,60-U51D69L59R61,61-U52D70L60R62,62-U53D71L61,63-U54D72R64,64-U55D73L63R65,65-U56D74L64R66,66-U57D75L65R67,67-U58D76L66R68,68-U59D77L67R69,69-U60D78L68R70,70-U61D79L69R71,71-U62D80L70,72-U63R73,73-U64L72R74,74-U65L73R75,75-U66L74R76,76-U67L75R77,77-U68L76R78,78-U69L77R79,79-U70L78R80,80-U71L79,<JDA>:
	x

#Aldous-Broder algorithm-ish :D
# in hindsight this performs way tooo many random steps to find that isolated corner cell for ex that hasn't been visited yet: need faster algorithm!
	#mark each cell's visitation as False
	s:[0-9]+:&,F:g
	#set first cell as current cell, would be nice if this was randomized to be anywhere on the 4 sides, and mark its visitation to True
	s:#0,F#:#C0,T#:
	:loop_ABa
		##own RNG using LCG
		#x
		#s:<GCL>:#result_lcgp_m&:
		#b lcg_pos
		:result_lcgp_m
			#s:#[^<]+(<GCL>)( [0-9]+ [0-9]+ [0-9]+)(<ADJ>):\2\1\2\3:
		#x;G

		#system RNG
		G;x
		s:.*:seq 0 999|shuf|head -n 1:e
		x;G
		s:(<LCG>)[0-9]+( [0-9]+ [0-9]+ [0-9]+)(<GCL>.*<JDA>)\n([0-9]+):\1\4\2\3:
		x;G
		s:.*(<LCG>):\1:
		x

		#pick a random neighbour
		s:(C)([0-9]+)(,.*<ADJ>.*,)\2(-):\1\2\3\2\4@:
			#2 neighbours: corner cells
			/@((D[0-9]+R[0-9]+)|(D[0-9]+L[0-9]+)|(U[0-9]+R[0-9]+)|(U[0-9]+L[0-9]+)),/{
				/<LCG>[0-9]*[02468] /s:([#=])([0-9]+)(,.*<ADJ>.*@)([UDLR])\2([UDLR]):\1N\4\2\3\4\2\5:
				/<LCG>[0-9]*[13579] /s:([#=])([0-9]+)(,.*<ADJ>.*@[UDLR][0-9]+)([UDLR])\2(,):\1N\4\2\3\4\2\5:
			}
			#3 neighbours: side cells, except corners
			/@((D[0-9]+L[0-9]+R[0-9]+)|(U[0-9]+D[0-9]+R[0-9]+)|(U[0-9]+D[0-9]+L[0-9]+)|(U[0-9]+L[0-9]+R[0-9]+)),/{
				/<LCG>[0-9]*[012] /s:([#=])([0-9]+)(,.*<ADJ>.*@)([UDLR])\2([UDLR]):\1N\4\2\3\4\2\5:
				/<LCG>[0-9]*[3456] /s:([#=])([0-9]+)(,.*<ADJ>.*@[UDLR][0-9]+)([UDLR])\2([UDLR]):\1N\4\2\3\4\2\5:
				/<LCG>[0-9]*[789] /s:([#=])([0-9]+)(,.*<ADJ>.*@[UDLR][0-9]+[UDLR][0-9]+)([UDLR])\2(,):\1N\4\2\3\4\2\5:
			}
			#4 neighbours: interior cells
			/@U[0-9]+D[0-9]+L[0-9]+R[0-9]+,/{
				/<LCG>[0-9]*[01] /s:([#=])([0-9]+)(,.*<ADJ>.*@)([UDLR])\2([UDLR]):\1N\4\2\3\4\2\5:
				/<LCG>[0-9]*[234] /s:([#=])([0-9]+)(,.*<ADJ>.*@[UDLR][0-9]+)([UDLR])\2([UDLR]):\1N\4\2\3\4\2\5:
				/<LCG>[0-9]*[567] /s:([#=])([0-9]+)(,.*<ADJ>.*@[UDLR][0-9]+[UDLR][0-9]+)([UDLR])\2([UDLR]):\1N\4\2\3\4\2\5:
				/<LCG>[0-9]*[89] /s:([#=])([0-9]+)(,.*<ADJ>.*@[UDLR][0-9]+[UDLR][0-9]+[UDLR][0-9]+)([UDLR])\2(,):\1N\4\2\3\4\2\5:
			}
		s:\n<LCG>.*::
		/N[UDLR][0-9]+,F/{
			#remove (in this case convert from # to =) the wall between the current cell and the neighbour
			/NL/s:#(C):=\1:
			/NR/s:#(N):=\1:
			/N[UD]/{
				s:N[UD]([0-9]+),.*$:&;<MOD>\1 9#result_mdp_m<DOM>:
				b mod_pos
				:result_mdp_m
					s:<MOD>([0-9]+)[^<]+<DOM>$:<ADD>2 \1 \1#result_ap_m<DDA>:
				b add_pos
				:result_ap_m
					s:<ADD>([0-9]+)[^<]+<DDA>$:<SEQ>1-\1#result_sp_m2<QES>:
				b seq_pos
				:result_sp_m2
					s:(<SEQ>) ?[0-9]+:@\1:
				/<SEQ>#/!b result_sp_m2
				s:<SEQ>.*$::
			}
			/NU/{
				s:N[^\n]+\n:&@:
				b shift
			}
			/ND/{
				s:C[^\n]+\n:&@:
				:shift
					s:(@)([#=]):\2\1:
					s:(;)@:\1:
				/;$/!b shift
				s:[#=]@:=:
				s:;$::
			}
			#mark neighbour's visitation to True
			s:(N[UDLR][0-9]+),F:\1,T:
		}
		#set the neighbour as current cell
		s:C::
		s:N[UDLR]:C:
p#debug
	/F/b loop_ABa

#display
	:print
		s:C::
		s:([#=])(0):\1S\2:
		s:([#=])(80):\1E\2:
		s:\b([SE]?)[0-9]+,[TF]:\1 :g
		s:([SE]) :\1:g
		y:=: :
		#ANSI colors and fixed height/width ratio on terminal (h~=2w)
			s: :&&:g
			s:#:\o033[30;40m  \o033[0m:g
			s:[SE]:\o033[31;41m  \o033[0m:g
		p
b EOS


:user_redirects
	/##result_sp_m<QES>/b result_sp_m
	/##result_lcgp_m<GCL>/b result_lcgp_m
	/##result_mdp_m<DOM>/b result_mdp_m
	/##result_ap_m<DDA>/b result_ap_m
	/##result_sp_m2<QES>/b result_sp_m2
	#/##result_debug<[^>]+>/b result_debug
b EOS


:debug
	#test if GNU extensions are supported
	1v
	#s:.*:<LCG>&#result_debug<GCL>:;b lcg_pos
	#:result_debug
		#s:<[^>]+>([^#]+)[^<]+<[^>]+>:\1:p
	#b EOS
b main


######################### MATH / UTILS ######################### (most of it was written before cg, used as my standard library :)

#4: <LCG>seed a c m#label<GCL> -> <LCG>new_seed##label<GCL>
:lcg_pos
	s:(<LCG>)([0-9]+ [0-9]+):\1<MULT>\2#result_mpp_lcgp<TLUM>:
	b mult_pos
	:result_mpp_lcgp
		s:(<LCG>)<MULT>([0-9]+)[^<]+<TLUM>:\1\2:
	s:(<LCG>)([0-9]+) ([0-9]+):\1<ADD>\3 \2#result_ap_lcgp<DDA>:
	b add_pos
	:result_ap_lcgp
		s:(<LCG>)<ADD>([0-9]+)[^<]+<DDA>:\1\2:
	s:(<LCG>)([0-9]+ [0-9]+):\1<MOD>\2#result_mdp_lcgp<DOM>:
	b mod_pos
	:result_mdp_lcgp
		s:(<LCG>)<MOD>([0-9]+)[^<]+<DOM>:\1\2#:
b redirect


#2: <INTERSECTION>3 4 5,2 3 4 5 6 7 8#label<NOITCESRETNI> -> <INTERSECTION>1##label<NOITCESRETNI>
:intersection_pos
	s:(<INTERSECTION>)([^,]+):\1!\2 :
	:loop_isp
		/<INTERSECTION>[^!]+!,/{
			s:(<INTERSECTION>)[^#]+:\10:
			b print_isp
		}
		/<INTERSECTION>[^!]*!([0-9]+) [^,]*,[^#]*\b\1\b/{
			s:(<INTERSECTION>)[^#]+:\11:
			b print_isp
		}
		s:(<INTERSECTION>[^!]*)!([0-9]+ ):\1\2!:
	b loop_isp
	:print_isp
		s:#[^<]+<NOITCESRETNI>:#&:
b redirect


#2: <SUBSET>3 4 5,2 3 4 5 6 7 8#label<TESBUS> -> <SUBSET>1##label<TESBUS>
:subset_pos
	s:(<SUBSET>)([^,]+):\1!\2 :
	:loop_ssp
		/<SUBSET>[^!]+!,/{
			s:(<SUBSET>)[^#]+:\11:
			b print_ssp
		}
		/<SUBSET>[^!]*!([0-9]+) [^,]*,[^#]*\b\1\b/{
			s:(<SUBSET>[^!]*)!([0-9]+ ):\1\2!:
			b loop_ssp
		}
	s:(<SUBSET>)[^#]+:\10:
	:print_ssp
		s:#[^<]+<TESBUS>:#&:
b redirect


#1+: <SEQ>3-5,2-8,7-7#label<QES> -> <SEQ>3 4 5,2 3 4 5 6 7 8,7##label<QES>
:seq_pos
	s:<SEQ>:&,:
	b next_sp
	:loop_sp
		/,([0-9]+)<INC>\1#[^<]+<CNI>[^<]+<QES>/b next_sp
		b incr_pos
		:result_ip_sp
			s:(<INC>)([0-9]+)#([^<]+<CNI>)([^,#]+)([^<]+<QES>):\1\2\3\4 \2\5:
	b loop_sp
	:next_sp
		s:,[0-9]+<INC>[^<]+<CNI>([^<]+<QES>):,\1:
		/,[0-9]+-[0-9]+[,#][^<]+<QES>/!b print_sp
		s:,([0-9]+)-([0-9]+)([,#][^<]+<QES>):,\2<INC>\1#result_ip_sp<CNI>\1\3:
	b loop_sp
	:print_sp
		s:(<SEQ>),([^#]+)(#[^<]+<QES>):\1\2#\3:
b redirect


#1+: <MAX>1 2 3#label<XAM> -> <MAX>3##label<XAM>
:max_pos
	s:(<MAX>)([0-9]+#):\10 \2:
	s: ?([0-9]+) ([0-9]+)(#[^<]+<XAM>):A\1,\1B\2,\2M\2\3:
	:loop_Mp
		/,0[BM][^<]+<XAM>/b next_Mp
		s:,([0-9]+)(B[^<]+<XAM>):,<DEC>\1#result_dp_Mp1<CED>\2:
		b decr_pos
		:result_dp_Mp1
			s:,<DEC>([0-9]+)#[^<]+<CED>(B[^<]+<XAM>):,\1\2:
		s:,([0-9]+)(M[^<]+<XAM>):,<DEC>\1#result_dp_Mp2<CED>\2:
		b decr_pos
		:result_dp_Mp2
			s:,<DEC>([0-9]+)#[^<]+<CED>(M[^<]+<XAM>):,\1\2:
	b loop_Mp
	:next_Mp
		s:A([0-9]+),[1-9][0-9]*B[0-9]+,0M[0-9]+([^<]+<XAM>):A0,0B0,0M\1\2:
		/<MAX>A/b print_Mp
		s:B[0-9]+,[0-9]+M([0-9]+)([^<]+<XAM>):B\1,\1M\1\2:
		s: ?([0-9]+)A[0-9]+,[0-9]+:A\1,\1:
	b loop_Mp
	:print_Mp
		s:A[0-9]+,[0-9]+B[0-9]+,[0-9]+M([0-9]+):\1:
		s:#[^<]+<XAM>:#&:
b redirect


#1+: <MIN>1 2 3#label<NIM> -> <MIN>1##label<NIM>
:min_pos
        s:(<MIN>)([0-9]+)#:\1\20 \2#:
	s:(<MIN>0)0:\1:
        s: ?([0-9]+) ([0-9]+)(#[^<]+<NIM>):A\1,\1B\2,\2m\2\3:
        :loop_mp
                /,0[Bm][^<]+<NIM>/b next_mp
#compare lengths first before the dec method
                s:,([0-9]+)(B[^<]+<NIM>):,<DEC>\1#result_dp_mp1<CED>\2:
                b decr_pos
                :result_dp_mp1
                        s:,<DEC>([0-9]+)#[^<]+<CED>(B[^<]+<NIM>):,\1\2:
                s:,([0-9]+)(m[^<]+<NIM>):,<DEC>\1#result_dp_mp2<CED>\2:
                b decr_pos
                :result_dp_mp2
                        s:,<DEC>([0-9]+)#[^<]+<CED>(m[^<]+<NIM>):,\1\2:
        b loop_mp
        :next_mp
                s:A([0-9]+),0B[0-9]+,[1-9][0-9]*m[0-9]+([^<]+<NIM>):A0,0B0,0m\1\2:
                /<MIN>A/b print_mp
                s:B[0-9]+,[0-9]+m([0-9]+)([^<]+<NIM>):B\1,\1m\1\2:
                s: ?([0-9]+)A[0-9]+,[0-9]+:A\1,\1:
        b loop_mp
        :print_mp
                s:A[0-9]+,[0-9]+B[0-9]+,[0-9]+m([0-9]+):\1:
                s:#[^<]+<NIM>:#&:
b redirect


#2: <MOD>5 2#label<DOM> -> <MOD>1##label<DOM>
:mod_pos
	s:#[^>]+<DOM>:,0 0,0&:
	:loop_mdp_1
		s:(<MOD>[0-9]+ [0-9]+,)([0-9]+) [0-9]+,[0-9]+:\1\2 \2:
		s:(<MOD>[0-9]+ )([0-9]+),([0-9]+):\1\2,<ADD>\2 \3#result_ap_mdp<DDA>:
		b add_pos
		:result_ap_mdp
			s:<ADD>([0-9]+)[^>]+<DDA>([^>]+<DOM>):\1\2:
		s:<MOD>([0-9]+) [0-9]+,([0-9]+) [0-9]+:&<MIN>\1 \2#result_mp_mdp<NIM>:
		b min_pos
		:result_mp_mdp
			s:<MIN>([0-9]+)[^>]+<NIM>:,\1:
	/<MOD>([0-9]+) [0-9]+,[0-9]+ [0-9]+,\1/!b loop_mdp_1
	/<MOD>([0-9]+) [0-9]+,\1/ s:(<MOD>[0-9]+ [0-9]+),([0-9]+) [0-9]+:\1,\2 \2:
	s:(<MOD>[0-9]+) [0-9]+,[0-9]+ ([0-9]+),[0-9]+:\1,\2 0:
	:loop_mdp_2
		/<MOD>([0-9]+),\1/b print_mdp
		s:(<MOD>)([0-9]+):\1<DEC>\2#result_dp_mdp<CED>:
		b decr_pos
		:result_dp_mdp
			s:<DEC>([0-9]+)[^>]+<CED>:\1:
		s:(<MOD>[0-9]+,[0-9]+ )([0-9]+):\1<INC>\2#result_ip_mdp<CNI>:
		b incr_pos
		:result_ip_mdp
			s:<INC>([0-9]+)[^>]+<CNI>:\1:
	b loop_mdp_2
	:print_mdp
		s:(<MOD>)[0-9]+,[0-9]+ ([0-9]+):\1\2#:
b redirect


#1+: <MULT>2 3 4#label<TLUM> -> <MULT>24##label<TLUM>
:mult_pos
	s:(<MULT>)([0-9]+#):\11 \2:
	:loop_mpp
		/ ?\b0\b ([0-9]+)(#[^<]+<TLUM>)/{
			s:(<MULT>)[^#]+:\1 0:
			b print_mpp
		}
		s: ?([0-9]+) ([0-9]+)(#[^<]+<TLUM>):<ADD>\1 \2#result_ap_mpp<DDA>\3:
		s:(<ADD>)([0-9]+) ([^<]+<DDA>[^<]+<TLUM>):\1<SEQ>1-\2#result_sp_mpp<QES>\3:
		b seq_pos
		:result_sp_mpp
			s:(<SEQ>)[0-9]+ ?([^#]*[^<]+<QES>)([0-9]+):\3 \1\2\3:
		/<SEQ>#/!b result_sp_mpp
		s: <SEQ>[^<]+<QES>[0-9]+::
		b add_pos
		:result_ap_mpp
			s:<ADD>([^#]+)[^<]+<DDA>: \1:
	/<MULT>[0-9]+ /b loop_mpp
	:print_mpp
		s:(<MULT>) :\1:
		s:#[^<]+<TLUM>:#&:
b redirect


#1+: <ADD>1 2 3#label<DDA> -> <ADD>6##label<DDA>
:add_pos
	s:(<ADD>)([0-9]+#):\10 \2:
	s: ?([0-9]+) ([0-9]+)(#[^<]+<DDA>):<DEC>\1#result_dp_ap<CED><INC>\2#result_ip_ap<CNI>\3:
	:loop_ap
		/<DEC>0/b next_ap
		b decr_pos
		:result_dp_ap
			s:#([^<]+<CED>):\1:
		b incr_pos
		:result_ip_ap
			s:#([^<]+<CNI>):\1:
	b loop_ap
	:next_ap
		/<ADD><DEC>/b print_ap
		s: ?([0-9]+)(<DEC>)0:\2\1:
	b loop_ap
	:print_ap
		s:<DEC>0#result_dp_ap<CED><INC>([0-9]+)#result_ip_ap<CNI>(#[^<]+<DDA>):\1#\2:
b redirect


#1: <DEC>10#label<CED> -> <DEC>9##label<CED>
:decr_pos
	:zeros
		s:0(@*)(#[^<]+<CED>):@\1\2:
	t zeros
	s:9(@*)(#[^<]+<CED>):8\1\2:;t print_dp
	s:8(@*)(#[^<]+<CED>):7\1\2:;t print_dp
	s:7(@*)(#[^<]+<CED>):6\1\2:;t print_dp
	s:6(@*)(#[^<]+<CED>):5\1\2:;t print_dp
	s:5(@*)(#[^<]+<CED>):4\1\2:;t print_dp
	s:4(@*)(#[^<]+<CED>):3\1\2:;t print_dp
	s:3(@*)(#[^<]+<CED>):2\1\2:;t print_dp
	s:2(@*)(#[^<]+<CED>):1\1\2:;t print_dp
	s:1(@*)(#[^<]+<CED>):0\1\2:;t print_dp
	:print_dp
		s:(<DEC>)0@:\1@:
		:loop_dp
			s:(<DEC>[^#]*)@:\19:
		/<DEC>[^#]*@/b loop_dp
		s:#[^<]+<CED>:#&:
b redirect


#1: <INC>9#label<CNI> -> <INC>10##label<CNI>
:incr_pos
	:nines
		s:9(@*)(#[^<]+<CNI>):@\1\2:
	t nines
	s:0(@*)(#[^<]+<CNI>):1\1\2:;t print_ip
	s:1(@*)(#[^<]+<CNI>):2\1\2:;t print_ip
	s:2(@*)(#[^<]+<CNI>):3\1\2:;t print_ip
	s:3(@*)(#[^<]+<CNI>):4\1\2:;t print_ip
	s:4(@*)(#[^<]+<CNI>):5\1\2:;t print_ip
	s:5(@*)(#[^<]+<CNI>):6\1\2:;t print_ip
	s:6(@*)(#[^<]+<CNI>):7\1\2:;t print_ip
	s:7(@*)(#[^<]+<CNI>):8\1\2:;t print_ip
	s:8(@*)(#[^<]+<CNI>):9\1\2:;t print_ip
	:print_ip
		s:(<INC>)@:\11@:
		:loop_ip
			s:(<INC>[^#]*)@:\10:
		/<INC>[^#]*@/b loop_ip
		s:#[^<]+<CNI>:#&:
b redirect


:redirect
	b library_redirects
	:continue_redirects
	b user_redirects


:library_redirects
	/##result_mpp_lcgp<TLUM>/b result_mpp_lcgp
	/##result_ap_lcgp<DDA>/b result_ap_lcgp
	/##result_mdp_lcgp<DOM>/b result_mdp_lcgp
	/##result_dp_ap<CED>/b result_dp_ap
	/##result_dp_Mp1<CED>/b result_dp_Mp1
	/##result_dp_Mp2<CED>/b result_dp_Mp2
	/##result_dp_mp1<CED>/b result_dp_mp1
	/##result_dp_mp2<CED>/b result_dp_mp2
	/##result_ap_mdp<DDA>/b result_ap_mdp
	/##result_mp_mdp<NIM>/b result_mp_mdp
	/##result_dp_mdp<CED>/b result_dp_mdp
	/##result_ip_mdp<CNI>/b result_ip_mdp
	/##result_sp_mpp<QES>/b result_sp_mpp
	/##result_ap_mpp<DDA>/b result_ap_mpp
	/##result_ip_ap<CNI>/b result_ip_ap
	/##result_ip_sp<CNI>/b result_ip_sp
b continue_redirects


:EOS
	#End Of Script (mainly used to skip over remaining code, when needed)
	Q
